/*
README, as well as source code, can be found at https://git.420joos.dev/ost/we1/testat-rockpaperscissoirs/-/blob/dev/README.md
 */

import {getRankings, evaluateHand, createUserIfNotExists, isConnected, setConnected} from './game-service.js';

const loginScreen = document.querySelector('#login');
const gameScreen = document.querySelector('#game');

const options = document.querySelector('#options');
const waitLabel = document.querySelector('#wait-label');

const changeModeButton = document.querySelector('#change-online-mode-button');
const loginForm = document.querySelector('#login-form');
const usernameInput = document.querySelector('#username-input');
const rankingTable = document.querySelector('#ranking-table');

const playerHandLabel = document.querySelector('#player-hand');
const opponentHandLabel = document.querySelector('#opponent-hand');

const mainMenuButton = document.querySelector('#main-menu');
const historyTableBody = document.querySelector('#history');
const usernameDisplay = document.querySelector('#username-display');

let username = '';

const notWonClass = 'lost-draw-color';
const wonClass = 'win-color';
const visibleClass = 'visible';
const hiddenClass = 'hidden';
const optionsDisabledClass = 'options-disabled';
const optionsEnabledClass = 'options-enabled';
const modeChangeDisabledClass = 'mode-change-disabled';
const modeChangeEnabledClass = 'mode-change-enabled';

function addToHistory(userHand, opponentHand, winText) {
    historyTableBody.innerHTML = `<tr><td>${winText}</td>\t<td>${userHand}</td>\t<td>${opponentHand}\n</td></tr>${historyTableBody.innerHTML}`;
}

function handleOpponentPicked(userHand, opponentHand, gameEval) {
    opponentHandLabel.innerText = opponentHand;

    let winText;
    // eslint-disable-next-line @web-and-design/wed/use-action-map
    if (gameEval < 0) {
        winText = 'Lost';
        opponentHandLabel.classList.replace(notWonClass, wonClass);
    } else if (gameEval > 0) {
        winText = 'Won';
        playerHandLabel.classList.replace(notWonClass, wonClass);
    } else {
        winText = 'Draw';
    }

    addToHistory(userHand, opponentHand, winText);

    let timeout = 3;
    waitLabel.innerText = 'Next round starts in soon';
    const countdown = setInterval(() => {
        if (timeout > 0) {
            waitLabel.innerText = `Next round starts in ${timeout}`;
            timeout--;
        } else {
            clearTimeout(countdown);
            waitLabel.innerText = '\n';
            options.classList.replace(optionsDisabledClass, optionsEnabledClass);
        }
    }, 1000);
}

function sanitizePlayername(playerName) {
    return playerName.replaceAll('<', '&lt;').replaceAll('>', '&gt;');
}

function showRankings(rankings) {
    let rankingsHtml = '';
    for (const ranking of rankings) {
        rankingsHtml += `<tr><td>${ranking.rank}</td><td>${ranking.wins}</td><td>`;
        for (let i = 0; i < ranking.players.length - 1; i++) {
            const playerName = sanitizePlayername(ranking.players[i]);
            rankingsHtml += `${playerName}, `;
        }
        const playerName = sanitizePlayername(ranking.players[ranking.players.length - 1]);
        rankingsHtml += `${playerName}</td></tr>`;
    }

    rankingTable.innerHTML = rankingsHtml;
}

function updateRankings(rankings) {
    if (changeModeButton.classList.contains(modeChangeDisabledClass)) {
        changeModeButton.classList.replace(modeChangeDisabledClass, modeChangeEnabledClass);
    }
    showRankings(rankings);
}

function onOptionClick(event) {
    const userHand = event.target.innerText;
    playerHandLabel.innerText = userHand;
    opponentHandLabel.innerText = '\n';

    // eslint-disable-next-line @web-and-design/wed/use-action-map
    if (playerHandLabel.classList.contains(wonClass)) {
        playerHandLabel.classList.replace(wonClass, notWonClass);
    }

    if (opponentHandLabel.classList.contains(wonClass)) {
        opponentHandLabel.classList.replace(wonClass, notWonClass);
    }

    options.classList.replace(optionsEnabledClass, optionsDisabledClass);

    evaluateHand(username, userHand, ({
        systemHand,
        gameEval,
    }) => handleOpponentPicked(userHand, systemHand, gameEval));
}

// used to switch between login screen and game screen
function setScreenVisible(enablePlayScreen) {
    if (enablePlayScreen) {
        // eslint-disable-next-line @web-and-design/wed/use-action-map
        if (loginScreen.classList.contains(visibleClass)) {
            loginScreen.classList.remove(visibleClass);
        }

        if (gameScreen.classList.contains(hiddenClass)) {
            gameScreen.classList.remove(hiddenClass);
        }

        loginScreen.classList.add(hiddenClass);
        gameScreen.classList.add(visibleClass);
    } else {
        // eslint-disable-next-line @web-and-design/wed/use-action-map
        if (loginScreen.classList.contains(hiddenClass)) {
            loginScreen.classList.remove(hiddenClass);
        }

        if (gameScreen.classList.contains(visibleClass)) {
            gameScreen.classList.remove(visibleClass);
        }

        loginScreen.classList.add(visibleClass);
        gameScreen.classList.add(hiddenClass);
    }
}

function resetGameScreen() {
    historyTableBody.innerText = '';

    playerHandLabel.classList.add(notWonClass);
    opponentHandLabel.classList.add(notWonClass);

    playerHandLabel.innerText = '';
    opponentHandLabel.innerText = '';
}

function onLoginClick(e) {
    e.preventDefault();

    if (usernameInput.value.length === 0) {
        return false;
    }
    username = usernameInput.value;
    usernameDisplay.innerText = username;
    resetGameScreen();

    setScreenVisible(true);

    // reset box colors & content

    createUserIfNotExists(username);
    return false;
}

function onMainMenuClick() {
    setScreenVisible(false);
    getRankings(showRankings);
}

function updateOnlineButton() {
    const preText = 'Change to';
    if (isConnected()) {
        changeModeButton.innerText = `${preText} local`;
    } else {
        changeModeButton.innerText = `${preText} online`;
    }
}

function triggerOnlineState() {
    changeModeButton.classList.replace(modeChangeEnabledClass, modeChangeDisabledClass);

    setConnected(!isConnected());
    updateOnlineButton();
    getRankings(updateRankings);
}

options.addEventListener('click', (event) => onOptionClick(event));
loginForm.addEventListener('submit', onLoginClick);
mainMenuButton.addEventListener('click', onMainMenuClick);
changeModeButton.addEventListener('click', triggerOnlineState);

window.addEventListener('DOMContentLoaded', () => setScreenVisible(false));
window.addEventListener('DOMContentLoaded', () => getRankings(updateRankings));
window.addEventListener('DOMContentLoaded', updateOnlineButton);
